.include "m8def.inc"
.device atmega8
;--------------------------------------------------------------------------------
;Register, frei sind noch r3,r16
.def	S		=r1		;SRegister


.def	z60		=r8		;Konstante
.def	z10		=r7
.def	z255	=r2
.def	z16		=r0

.def	temp	=r18	;Trash
.def	temp1	=r19
.def	temp2	=r20
.def	temp3	=r21

.def	itemp	=r17	;ISR
.def	icnt	=r6
.def	beep	=r9
.def	beepl	=r10

.def	enc		=r22	;Drehencoder
.def	richtung=r4
.def	taster	=r15

.def	speed	=r5
.def	upm		=r14
.def	upm01	=r12
.def	ti		=r13
.def	upm16	=r11

.equ	i2cPort	=DDRD	;I2C Adressen
.equ	i2cPin	=PinD
.equ	sda	=	DDD0	;wird ber Datenrichtungsregister (open Collector) geproggt
.equ	scl	=	DDD1
.equ	lm75	=$9e	;LM75 Adresse schreiben (lesen+1) Innentemp.


.macro	print
		ldi		zl,low(@0*2)		;Text auf LCD
		ldi		zh,high(@0*2)
		rcall	tx1
.endmacro

.macro	cur									;Cursor setzen x,y
		ldi		temp1,1<<7|@0+$40*@1		;x=0..15, Y=0..1
		rcall	BefOut								
.endmacro
.cseg
		rjmp	Reset
.org	OC1Aaddr
		rjmp	isr	

;--------------------------------------------------------------------
;Genau alle 1mS

isr:	in		S,sreg
		push	temp

;--------------------------------------------------------------------
;Motorumdrehung ermitteln
		dec		ti
		brne	isr0
		mov		ti,z16				;Ab hier .. alle 16 mS
		mov		itemp,upm
		inc		upm01
		brne	isr01
;Fehler
		dec		upm01
		mov		upm16,upm01
isr01:	clc	
		sbic	pinb,4				;berspringe wenn gedrckt
		sec
isr02:	rol		itemp
		cbr		itemp,0b11111000	;nach der fallenden Flanke muss
		cpi		itemp,0b100			;die Null 2 ISRs (=32 mS) anliegen (entprellen bei Mikroschalter)
		brne	isr03	
		mov		upm16,upm01			;eine Umdrehung hat upm01*16mS gedauert
		clr		upm01
isr03:	mov		upm,itemp

;UP/M= 60000/upm01*16

;--------------------------------------------------------------------		
		mov		itemp,taster		;Den Taster am Encoder entprellen
		clc
		sbic	pinb,2				
		sec
		rol		itemp				;Wenn gedrckt, ist eine Null im Carry
		cpi		itemp,0b10000000	;die Null muss 7 ISRs (=112 mS) anliegen, der Taster war also vor 8 ISRs noch high
		brne	isr04				;dann ist der Taster "sicher" prellfrei
		ser		temp
		sts		switch,temp			;Switch auf $ff, wird im main gelscht.
isr04:	mov		taster,itemp
;--------------------------------------------------------------------
;Uhr weiterzhlen
isr0:	dec		icnt				;alle 10 mS die Uhr hochzhlen
		brne	isr1
		mov		icnt,z10

		lds		itemp,msekunden		;Millisekunden/10 hochzhlen
		cpi		itemp,99			;Bei berlauf inc Sekunden
		breq	incsek
		inc		itemp
		sts		msekunden,itemp
		rjmp	isr1
incsek:	clr		itemp
		sts		msekunden,itemp		;Sekunden dito
		lds		itemp,sekunden
		cpi		itemp,59
		breq	incmin
		inc		itemp
		sts		sekunden,itemp
		rjmp	isr1
incmin:	clr		itemp				;Minuten
		sts		sekunden,itemp
		lds		itemp,minuten
		cpi		itemp,59
		breq	incstu
		inc		itemp
		sts		minuten,itemp
		rjmp	isr1
incstu:	clr		itemp				;Stunden
		sts		minuten,itemp
		lds		itemp,stunden
		inc		itemp
		sts		stunden,itemp
;Angezeigt wird nur MM:SS, bei grsserem LCD knnte man allerdings mehr anzeigen
;-------------------------------------------------------------------
;Buzzer an pd2 und pd3
isr1:	tst		beep
		breq	isr2
		dec		beepl
		brne	isr1a
		dec		beep
isr1a:	sbrs	beepl,0	
		sbi		portd,2
		sbrc	beepl,0
		cbi		portd,2
		sbrs	beepl,0	
		cbi		portd,3
		sbrc	beepl,0
		sbi		portd,3
;-------------------------------------------------------------------
;Encoder auslesen, ist verbunden mit pb0 und pb1

isr2:	in		itemp,pinb
		ror		itemp			;pinb0 ins Carry...
		rol		enc				;.. und in bit0 
		ror		itemp			;pinb1 ins Carry
		rol		enc				;und nochmal schieben
		andi	enc,$0f			;das Bitmuster der letzten ISR steht nun in bit2 und 3
		cpi		enc,0b1011		;bei den folgenden 4 kombis wurde nach links gedreht.
		breq	links
		cpi		enc,0b1101
		breq	links
		cpi		enc,0b0100
		breq	links
		cpi		enc,0b0010
		breq	links
		cpi		enc,0b0001		;rechts
		breq	rechts
 		cpi		enc,0b0111
 		breq	rechts
		cpi		enc,0b1110
		breq	rechts
		cpi		enc,0b1000
		breq	rechts
		rjmp	isr_exit		;bei allen anderen Kominationen wurde nicht gedreht,
								;oder Fehler (zu schnell), fast nicht mglich bei 1000Hz ISR-Freq.
rechts:	inc		richtung
		rjmp	isr_exit
links:	dec		richtung

		rjmp	isr_exit

;--------------------------------------------------------------------
;Alte Routine - ist zwar etwas krzer, braucht aber ein Register mehr
;		mov		itemp,alt		;Wert der letzten ISR
;		in		neu,pinb		;Neuen Wert holen
;		andi	neu,3			;Nur pinb0 und 1 auswerten
;		mov		alt,neu			;fr nchste ISR merken
;		cp		neu,itemp		;Alter und neuer Wert Vergleichen
;		breq	isr_exit		;Wenn gleich, wude nicht gedreht
;		swap	itemp			;alter Wert ins obere Nibble ..
;		or		neu,itemp		;.. und in einem Byte zusammenfassen
;bei folgenden Kombinationen wurde nach links gedreht
;		cpi		neu,$23			;von 10 nach 11
;		breq	links
;		cpi		neu,$31			;von 11 nach 01
;		breq	links
;		cpi		neu,$10			;von 01 nach 00
;		breq	links
;		cpi		neu,$02			;von 00 nach 10
;		breq	links
;Ansonsten wurde nach rechts gedreht
;-------------------------------------------------------------------
isr_exit:						;ISR Ende
		pop		temp
		out		sreg,S
		reti
;-------------------------------------------------------------------
;-------------------------------------------------------------------
;-------------------------------------------------------------------
;Init
Reset:	ldi  	temp,low(RAMEND)	;Stack
		out		SPL,temp			
		ldi  	temp,high(RAMEND)
		out		SPH,temp			

;---------------------------------------------------------------------
;Hardware PWM Timer2 anwerfen

		sbi		ddrb,3
		ldi		temp1,0<<WGM21|1<<WGM20|1<<COM21|1<<COM20|0<<CS22|0<<CS21|1<<CS20
		out		tccr2,temp1
		ldi		temp1,$ff
		out		ocr2,temp1

;---------------------------------------------------------------------
;Timer1 CTC anwerfen

		ldi		temp,high(8000-1)	;Vergleichswert =8000 ergibt
		out		OCR1AH,temp			;bei 8MHz genau 1000Hz (ISR alle 1mS)
		ldi		temp,low(8000-1)
		out		OCR1AL,temp
		ldi		temp,1<<WGM12|1<<CS10;Prescaler=1 Timer Compare ein	
		out		tccr1b,temp
		ldi		temp,1<<ocie1a		;Interrupt bei Timer Compare ein
		out		TIMSK,temp		

;------------------------------------------------------------------
;sonst. init

init:	ldi		temp,$ff			;Pullups PortB
		out		portb,temp
		ldi		temp,1<<2|1<<3		;Buzzer auf ausgang
		out		ddrd,temp
		out		portd,temp		;
		sbi		ddrb,5				;Relais PortB Ausgang
		cbi		portb,5
		rcall	load				;Eprom
		clr		temp				;Die Uhr auf 0
		sts		stunden,temp
		sts		minuten,temp
		sts		sekunden,temp
		sts		msekunden,temp
		ldi		temp,10				;Konstante init
		mov		z10,temp
		ldi		temp,60
		mov		z60,temp
		ldi		temp,255
		mov		z255,temp
		ldi		temp,16			
		mov		z16,temp
		clr		richtung			;Encoder auf 0
		mov		speed,z255			;Motor aus (invers)
		mov		upm01,z255
		clr		upm16
		mov		icnt,z10
		ldi		temp2,20
		sts		wtemp,temp2			;verzgerung fr LM75
		ldi		temp,0
		sts		switch,temp			;Taster auf 0
		sts		wrel,temp
		sei
;--------------------------------------------
;LCD Init
		rcall	lcd_init		
		ldi		temp1,0b01000000	;ins CG-Ram
		rcall	befout
		ldi		xl,64
		ldi		zl,low(Chars*2)		;Sonderzeichen fr den
		ldi		zh,high(Chars*2)	;Bargraph
movchars:lpm	temp1,z+
		rcall	chrout
		dec		xl
		brne	movchars
		ldi		temp1,1<<7			;wieder	auf Textmodus umschalten
		rcall	BefOut

;------------------------------------------------------------------
;Hauptprogramm

main:	rcall	clrs
		print	text
		cur		0,1
		print	text1
		cur	11,0					;Zeit
		lds		temp,minuten
		rcall	PrintZ99
		ldi		temp1,':'
		rcall	ascout
		lds		temp,sekunden
		rcall	PrintZ99
		clr		richtung
		clr		temp				;Wahl= Start
		sts		switch,temp

mloop:	lds		temp3,switch		;Taste gedrckt ?
		tst		temp3
		brmi	wahl				;Wenn ja, Sprung

		mov		temp3,richtung		;wurde am Encoder gedreht ?
		tst		temp3
		breq	mloop								
		brpl	ml1					;Rckwrts ?
		neg		temp3
ml1:	cpi		temp3,32			;8 Raster um wahl zu ndern
		brlo	mloop
		inc		temp
		cbr		temp,$fe			;
		tst		temp
		brne	ml2



ml3:	cur		0,1
		print	text1		;Text mit Marker <START>	
		clr		richtung
		rjmp	mloop

ml2:	cur		0,1
		print	text2
		clr		richtung
		rjmp	mloop
los1:	rjmp	los

wahl:	clr		temp3				;Taste rcksetzen
		sts		switch,temp3
		tst		temp
		breq	los1
;--------------------------------------------------------------
;Temperatur Sollwert einstellen
setup:	rcall	clrs
		print	text5				;"Schalttemperatur"
		clr		richtung		
		clr		temp				
		sts		switch,temp			;Taste lschen
		lds		temp,stemp
set1:	cur		0,1
		rcall	printtemp
		cur		7,1					;Cursor auf Zeile 2 Spalte 7 (pos. Bargraph)
		mov		temp1,temp
		lsl		temp1				;*2 damit die Bar voll wird
		dec		temp1				;-1, sonst berlauf der Bar
		rcall	bar
set2:	lds		temp3,switch		;Taste gedrckt ?
		tst		temp3
		brmi	set3				;Wenn ja, Sprung (bernehmen)
		tst		richtung
		breq	set2
		brpl	minustemp
		inc		Richtung
		cpi		temp,64*2			;64grad
		brcc	set2
		inc		temp
		rjmp	set1
minustemp:
		dec		Richtung
		cpi		temp,20*2+1			;20grad
		brcs	set2
		dec		temp
		rjmp	set1
set3:	sts		stemp,temp
;		rjmp	main


;-------------------------------------------------------------------

mspd:	rcall	hochfahren
		rcall	clrs				;Motorspeed
		print	text6				;"Schalttemperatur"
		clr		temp
		sts		switch,temp
msp1:	cur		0,1
		print	text7
		lds		temp,startspeed
		com		temp
		rcall	printziffer
		cur		8,1
		lds		temp1,startspeed
		com		temp1
		rcall	bar
		lds		temp,startspeed
msp2:	lds		temp3,switch		;Taste gedrckt ?
		tst		temp3
		brmi	mainx				;Wenn ja, Sprung (bernehmen)

		tst		richtung
		breq	msp2
		brmi	slow

		dec		richtung
		cpi		temp,255
		breq	msp2
		inc		temp
		sts		startspeed,temp
		out		ocr2,temp
		rjmp	msp1	




slow:	inc		richtung
		tst		temp
		breq	msp2
		dec		temp
		sts		startspeed,temp
		out		ocr2,temp
		rjmp	msp1	



mainx:	sts		startspeed,temp
		mov		speed,temp
		rcall	runterfahren


;
;
;Alarm "Erinnerungsbeep"



		clr		temp
		sts		switch,temp
		rcall	clrs
		print	text8
nxbep:	cur		0,1
		print	text9
		lds		temp,amin
		rcall	printz99
		ldi		temp1,':'
		rcall	ascout	
		lds		temp,asek
		rcall	printz99

msp3:	lds		temp3,switch		;Taste gedrckt ?
		tst		temp3
		brmi	mainy				;Wenn ja, Sprung (bernehmen)

		tst		richtung
		breq	msp3
		brpl	clkdwn

clkup:	inc		richtung
		lds		temp,asek
		cpi		temp,59
		breq	minup
		inc		temp
		sts		asek,temp
		rjmp	nxbep
minup:	lds		temp,amin
		cpi		temp,59
		breq	nxbep
		inc		temp
		sts		amin,temp	
		clr		temp
		sts		asek,temp
		rjmp	nxbep


clkdwn:	dec		richtung
		lds		temp,asek
		cpi		temp,0
		breq	mindwn
		dec		temp
		sts		asek,temp
		rjmp	nxbep
mindwn:	lds		temp,amin
		cpi		temp,0
		breq	nxbep
		dec		temp
		sts		amin,temp	
		ldi		temp,59
		sts		asek,temp
		rjmp	nxbep

mainy:	rcall	save
		clr		temp
		sts		stunden,temp
		sts		minuten,temp
		sts		sekunden,temp
		sts		msekunden,temp
		rjmp	main


;--------------------------------------------------------------

;---------------------------------------------------------------
los:	rcall	hochfahren			;Motor auf eingestellte Drehzahl fahren
		clr		richtung			;Drehencoder lschen
		clr		temp				;Stopuhr Rcksetzen
		sts		stunden,temp
		sts		minuten,temp
		sts		sekunden,temp
		sts		msekunden,temp
		sts		wrel,temp			;Verzgerung Relais
		rcall	clrs				;LCD lschen
start:	lds		temp3,switch		;Taste gedrckt ?
		tst		temp3
		breq	uhr					;Wenn nein Weiter zur Stopuhr
		rcall	runterfahren		;Motor aus
		cbi		portb,5				;Relais aus
		rjmp	main				;Zur auswahl
;Uhr
uhr:	rcall	home				;Cursor 0,0
		lds		temp,minuten
		rcall	PrintZ99			;MM:SS (werden in der ISR hochgezhlt) ausgeben
		ldi		temp1,':'
		rcall	ascout
		lds		temp,sekunden
		rcall	PrintZ99

		lds		temp,minuten		;Alarm ?
		lds		temp1,amin
		cp		temp,temp1			;Minuten vergleichen
		brne	therm				;Wenn ungleich Sprung
		lds		temp,sekunden
		lds		temp1,asek
		cp		temp,temp1			;Sekunden dito
		brne	therm
		ldi		temp,2
		mov		beep,temp			;Wenn gleich, Beepton (macht die ISR, die Dauer betrgt (temp)2*256 mS)
		
;----------------------------------------------------------------
;Temperatur
therm:	lds		temp,wtemp			;Verzgerung
		dec		temp
		sts		wtemp,temp	
		brne	notemp				;Lesen wir aus ?
		ldi		temp,150			;Den LM75 die nchsten 150 mal berspringen
		sts		wtemp,temp	

		cur		6,0
		rcall	tempe				;Den LM75 ber denI2C auslesen
		mov		temp,temp1			;Temp1 liefert dei Temperatur*2
		push	temp1
		rcall	printtemp			;Eigene Temperatur-Printroutine aufrufen
		cur		10,0
		ldi		temp1,'/'			;Slash auf LCD
		rcall	chrout
		lds		temp,stemp			;Schalttemperatur auf LCD
		rcall	printtemp			;Das sieht jetzt so aus: XX,X/YY,Y
									;						 ist/ soll
;----------------------------------------------------------------
;Relais fr Heizung

		lds		temp2,wrel			;Zhler	ca.alle 1sec.
		tst		temp2				;Nach einem Schaltvorgang
		breq	rel					;die Relaisroutine ca. 30 sec. lang berspringen
		dec		temp2				;Das verhindert das das Relais kollabiert.
		rjmp	rel1

rel:	pop		temp1				;Temperatur ist
		lds		temp,stemp			;Temperatur soll
		clr		temp2				;fr Wait nach dem Schaltvorgang
		cp		temp,temp1			;Ist und Sollwerte vergleichen
		brcs	relaus				;Wenn Istw. >, Relais aus

		sbis	portb,5				;War das Relais schon an ?
		ldi		temp2,30			;Wenn nein, die nchsten (ca.) 30 sek. aussetzen
		sbi		portb,5				;Relais ein
		rjmp	rel1

relaus:	sbic	portb,5
		ldi		temp2,30
		cbi		portb,5
rel1:	sts		wrel,temp2		

notemp:
;-------------------------------------------------------------
;Umdrehungen / Minute
;eine Umdrehung	hat upm16*16ms gedauert
;U/Min.=60000(ms)/upm16*16 oder gekrzt 3750/upm16
		cur		0,1					;Cursor in Zeile2
		print	text4				;UPM: auf Display
		ldi		YL,low(3750)	;16Bit Division (AN200 von Atmel) aufrufen
		ldi		YH,high(3750)
		mov		ZL,upm16
		clr		ZH
		rcall	div16u				;AN200 16Bit Division
		mov		temp,YL
		cp		upm16,z255			;Max.?
		brne	zok					;Bei nein, ist ok, Sprung
		ser		temp				;Wird dann null
zok:	inc		temp				;aufrunden
		rcall	PrintZiffer			;und aufs Display


test:	tst		richtung			;Am Encoder gedreht ?
		breq	sts2				;Bei nein, Sprung
		brmi	slower				;links ist um 1 langsamer
		dec		richtung			;rechts = schneller 
		cp		speed,z255			;Max.?
		breq	start1				;Bei ja Sprung
		inc		speed				;Speed + 1
sts1:	out		ocr2,speed			;Ins PWM Register
sts2:	cur		8,1					;Position des Bargraph
		mov		temp1,speed			
		com		temp1				;Invers ! 255= Motor aus / 0 = Motor volle Kanne
		rcall	bar					;Bar zeichnen
		rjmp	start				;Wieder von vorn

slower:	inc		richtung
		tst		speed
		breq	start1
		dec		speed
		rjmp	sts1
start1:	rjmp	start
;---------------------------------------------------------------------------------
;Unterprogramme
;---------------------------------------------------------------------------------
;Motor hochfahren (damit nichts berschwappt)
hochfahren:
		rcall	clrs
		print	text3
		lds		speed,startspeed
		mov		temp3,z255
		cp		speed,z255
		breq	hx
hf:		rcall	wait15
		cur		0,1
		mov		temp1,temp3
		com		temp1
		push	temp1
		mov		temp,temp1
		rcall	printziffer
		cur		8,1
		pop		temp1
		rcall	bar
		dec		temp3
		out		ocr2,temp3
		cp		speed,temp3
		brne	hf
hx:		ret
;----------------------------------------------------------------
;Motor runterfahren
runterfahren:
		sts		startspeed,speed
		mov		temp3,speed
rf:		cpi		temp3,$ff
		breq	rf1
		inc		temp3
		out		ocr2,temp3
		cur		8,1
		mov		temp1,temp3
		com		temp1
		rcall	bar
		rcall	wait5
		rcall	wait5
		rjmp	rf
rf1:	ret
;---------------------------------------------------------------------------------
;Bargraph 6 Zeichen mal 6 Pix = 36 Pix. Ein Byte muss durch 7 dividiert werden(255/7=36).
;Erwartet in temp1 ein wert von 0-255.

bar:	rcall	div7
		mov		yl,temp1
		ldi		temp1,7
		tst		yl
		brne	b000
		ldi		temp1,6
b000:	rcall	chrout
b00:	ldi		xl,6		;6 Zeichen ausgeben
b0:		cpi		yl,6
		brcs	endblock
		ldi		temp1,$ff	;ASC255 = Block
		rcall	chrout
		subi	yl,6
		dec		xl
		brne	b0
		rjmp	xx1
endblock:
		cpi		yl,5
		brne	b1
		ldi		temp1,$ff	;ASC255 = Block
		rjmp	b2
b1:		mov		temp1,yl
b2:		rcall	chrout
		clr		yl
		dec		xl
		breq	xx1
b3:		ldi		temp1,0
		rcall	chrout
		dec		xl
		brne	b3
xx1:	ldi		temp1,5
		rcall	chrout
		ret
;---------------------------------------------------------------------------------
;EEprom

Load:	ldi		xl,low(sram_start)		;Da sollen die Daten hin
		ldi		xh,high(sram_start)
		clr		temp1				;EEprom Adresse 0 = $000
		out		EEARH,temp1			;Adresse H
lo1:	rcall	ReadEE
		st		x+,temp
		inc		temp1				;Adresse +1
		cpi		temp1,$f			;Anzahl Bytes im EEprom
		brne	lo1
		ret			
;------------------------------------
;SRam ins EEprom schreiben
Save:	ldi		xl,low(sram_start)	;Quelldaten
		ldi		xh,high(sram_start)
		clr		temp1				;EEprom Adresse 0 = $000
		out		EEARH,temp1			;Adresse H
sa1:	ld		temp,x+				;Unit aus SRam
		rcall	WriteEE				;Burn Baby
		inc		temp1				;Adresse erhhen
		cpi		temp1,$f			;255 Bytes schreiben
		brne	sa1	
		ret			

;------------------------------------
;EEprom	lesen Adr.temp1 Daten temp (1 Byte)
ReadEE:								;Adresse 0x01
nxtEE:	sbic	EECR,EEWE			;Bereit ?
		rjmp	nxtEE
		out		EEARL,temp1			;Adresse setzen
		sbi		EECR,EERE			;ReadEnable in Controllregister
		in		temp,EEDR			;Daten Holen
		ret
;------------------------------------
;EEprom	schreiben Adr.temp1 Daten temp (1 Byte)
WriteEE:
		cli
WriteEE1:
		sbic	EECR,EEWE			;Bereit ?
		rjmp	WriteEE1
		out		EEARL,temp1			;Adresse
		out		EEDR,temp			;Daten
		cli
		sbi		EECR,EEMWE			;Master Write Enable
		sbi		EECR,EEWE			;Write Enable
		sei
		ret

;***************************************************************************
;*
;* "div8u" - 8/8 Bit Unsigned Division
;*
;* This subroutine divides the two register variables "dd8u" (dividend) and 
;* "dv8u" (divisor). The result is placed in "dres8u" and the remainder in
;* "drem8u".
;*  
;* Number of words	:14
;* Number of cycles	:97
;* Low registers used	:1 (drem8u)
;* High registers used  :3 (dres8u/dd8u,dv8u,dcnt8u)
;*
;***************************************************************************

;***** Subroutine Register Variables

.def	drem8u	=r29		;remainder
.def	dres8u	=r19		;result		(temp1)
.def	dd8u	=r19		;dividend (temp1)
;.def	dv8u	=r27		;divisor (fest auf 7)
.def	dcnt8u	=r28		;loop counter

;***** Code
;aus Atmel Appnote 200 ,teilt temp1 /7
div7:
div8u:	sub		drem8u,drem8u	;clear remainder and carry
		ldi		dcnt8u,9	;init loop counter
d8u_1:	rol		dd8u		;shift left dividend
		dec		dcnt8u		;decrement counter
		brne	d8u_2		;if done
		ret				;    return
d8u_2:	rol		drem8u		;shift dividend into remainder
		subi	drem8u,7	;remainder = remainder - divisor
		brcc	d8u_3		;if result negative
		subi	drem8u,-7	;    restore remainder
		clc					;    clear carry to be shifted into result
		rjmp	d8u_1		;else
d8u_3:	sec					;    set carry to be shifted into result
		rjmp	d8u_1


;---------------------------------------------------------------------------------
;***************************************************************************
;*
;* "div16u" - 16/16 Bit Unsigned Division
;*
;* This subroutine divides the two 16-bit numbers 
;* "dd8uH:dd8uL" (dividend) and "dv16uH:dv16uL" (divisor). 
;* The result is placed in "dres16uH:dres16uL" and the remainder in
;* "drem16uH:drem16uL".
;*  
;* Number of words	:19
;* Number of cycles	:235/251 (Min/Max)
;* Low registers used	:2 (drem16uL,drem16uH)
;* High registers used  :5 (dres16uL/dd16uL,dres16uH/dd16uH,dv16uL,dv16uH,
;*			    dcnt16u)
;*
;***************************************************************************
div16u:	clr		XL			;clear remainder Low byte
		sub		XH,XH		;clear remainder High byte and carry
		ldi		temp3,17		;init loop counter
d16u_1:	rol		YL			;shift left dividend
		rol		YH
		dec		temp3			;decrement counter
		brne	d16u_2		;if done
		ret					;    return
d16u_2:	rol		XL			;shift dividend into remainder
		rol		XH
		sub		XL,ZL		;remainder = remainder - divisor
		sbc		XH,ZH		;
		brcc	d16u_3		;if result negative
		add		XL,ZL		;    restore remainder
		adc		XH,ZH
		clc					;    clear carry to be shifted into result
		rjmp	d16u_1		;else
d16u_3:	sec					;    set carry to be shifted into result
		rjmp	d16u_1
;---------------------------------------------------------------------------------
;LCD - Routine 4-Bit
;AVR	-	LCD
;--------------
;PC0	-	DB4
;PC1	-	DB5
;PC2	-	DB6
;PC3	-	DB7
;PC4	-	RS
;PC5	-	E
;GND	-	R/W	(*nur reinschreiben und die max. Zeit abwarten)
;GND	-	DB0-DB3
;-------------------------------------------------------------
;LCD-Initialisierung
lcd_init:ldi	temp1,$3F
		out		DDRC,temp1			;PortC Ausgang
		rcall	wait15				;15ms warten
		ldi		temp3,3
		ldi		temp1,0b00000011	;Function Set 
		out		PORTC, temp1		;(Bits verschoben da 8 Bit Interface)	
_3mal:	sbi		PORTC,5				;3 * an Enable klingeln
		nop
		cbi		PORTC,5	
		rcall	wait5				;5ms warten
		dec		temp3				;Fertig ?
		brne	_3mal
		ldi		temp1,0b00000010	;Function Set (noch 8 Bit Modus)
		out		PORTC, temp1		;4 Bit-Interface 
		sbi		PORTC,5				;Enable
		nop
		cbi		PORTC,5
		rcall	wait5				;kurz warten
		ldi		temp1,0b00101000	;Function Set
		rcall	BefOut				;4Bit,2 Zeilen,5*7
		ldi		temp1,0b00001100	
		rcall	BefOut				;
		rcall	clrs				;Clear Display
		ldi		temp1,0b00000110	;Entry Mode Set
		rcall	BefOut				;
		ret
;-----------------------------------------------------------
;LCD-Befehl
chrout:	push	temp2				;Falls temp1 und 2 gebraucht werden
		push	temp1
		rcall	ascout
		pop		temp1
		pop		temp2
		ret


BefOut:	clr		temp2				;RS? NEIN
		sbrc	temp2,0				;berspringt immer
AscOut:	ldi		temp2,0b00010000	;Maske RS
		push	temp1				;Brauch ich nochmal
		swap	temp1				;erst mal Bit4-7       
		rcall	Nib1
		pop		temp1
Nib1:	andi	temp1,0b00001111	;den Rest wech	
		or		temp1,temp2			;setzt RS (oder hald net)
		out		PORTC, temp1
		sbi		PORTC,5				;Enable
		nop							;NOP muss
		cbi		PORTC,5				
		rcall	wait50				;braucht man normal nur beim 2. mal 
        ret                            
;------------------------------------------------------------
;String (Z) ausgeben
tx:		ldi		zl,low(text*2)		
		ldi		zh,high(text*2)
tx1:	lpm		temp1,z+
		tst		temp1
		breq	extx				;bei 0, fertig
		rcall	AscOut
		rjmp	tx1
;------------------------------------------------------------
home:	ldi		temp1,0b00000010	; Cursor Home
		sbrc	temp1,0				;springt immer
;------------------------------------------------------------
clrs:	ldi		temp1,0b00000001	;Display lschen
		rcall	BefOut
		rcall	wait5
extx:	ret
;------------------------------------------------------------
;Dezimalzahl (Byte) ausgeben
PrintZiffer:
		ldi		temp1,'0'-1		;Byte 3stellig ausgebeben 0..255
pz1:	inc		temp1			;Wobei die 100er Null ausgeblendet wird
		subi	temp,100
		brcc	pz1
		subi	temp,-100
		cpi		temp1,'0'
		brne	pz12
		ldi		Temp1,' '
pz12:	rcall	ChrOut
PrintZ99:
		ldi		temp1,'0'-1		;Byte 2stellig ausgeben 0..99
pz2:	inc		temp1
		subi	temp,10
		brcc	pz2
		subi	temp,-10
		rcall	ChrOut
		ldi		temp1,'0'
		add		temp1,temp
		rjmp	ChrOut
;temperatur auf	lcd
printtemp:
		push	temp			;Temperatur (*2) ausgeben xx,x
		lsr		temp			;Temperatur/2			
		rcall	printz99		;xx ausgeben
		ldi		temp1,','
		rcall	ChrOut			;, ausgeben
		ldi		temp1,'0'		;ASC "0" vorladen
		pop		temp			;kommt eine 0 oder 5 hinter das Komma ?		
		sbrc	temp,0			;Wenn Bit 0 gelscht "0" aufs LCD
		ldi		temp1,'5'		;ansonsten "5"
		rcall	ChrOut
		ldi		temp1,$df		;LCD Zeichen fr  Grad
		rjmp	ChrOut
;---------------------------------------------------------
;Warteschleifen fr LCD
wait50:	push	temp1		;50S
		ldi		temp1,132
wait:	dec		temp1
		brne	wait
		pop		temp1
		ret

wait15:	rcall	wait5		;15ms
		rcall	wait5

wait5:	push	temp1		;5ms
		ldi		temp1,100
warte:	rcall	wait50
		dec		temp1
		brne	warte
		pop		temp1
		ret
wait300:	push	temp1
			ldi		temp1,5
warte1:		rcall	wait15
			dec		temp1
			brne	warte1
			pop		temp1
			ret
;-------------------------------------------------------------------------------------
;Software I2C
;J.Woetzel 10/08, Version 1.0
;
;Es wird nur ein Register (temp) bentigt, das beim schreiben und beim lesen als bergaberegister
;fungiert. Beim schreiben dient das T-Flag als Error-Flag (T=1=Fehler). Die i2c_wait Routine
;ist fr 8MHz ausgelegt und kann leicht fr andere Frequenzen angepasste werden.
;-------------------------------------------------------------------------------------
;Macros fr I2C, ist normal nicht ntig aber bersichtlicher (4me)

.macro	scl0
		sbi	i2cPort,scl	;DDRD=Ausgang. Bewirkt fallende Flanke an SCL
.endmacro

.macro	scl1
		cbi	i2cPort,scl	;DDRD=Eingang. Macht SCL Hochohmig.
.endmacro				;Wird durch den Pullup an SCL auf High gezogen

.macro	sda0
		sbi	i2cPort,sda	;dito
.endmacro		

.macro	sda1
		cbi	i2cPort,sda
.endmacro

.macro	WState				;Testet ob Wait-State des Slaves
ws1:	sbis	i2cPin,scl	;Wenn der Slave die Clockleitung auf GND zieht..
		rjmp	ws1			;..warten
.endmacro
;-------------------------------------------------------------------------------------
;I2C-Startbedingung
i2c_start:
		scl0
		nop
		sda1
		nop
		scl1
		rcall	i2c_wait
		sda0
		rcall	i2c_wait
		scl0
		rcall	i2c_wait
		ret
;-------------------------------------------------------------------------------------
;I2C-Stopbedingung
i2c_stop:
		scl0
		sda0
		rcall	i2c_wait
		scl1
		rcall	i2c_wait
		sda1
		rcall	i2c_wait
		ret
;-------------------------------------------------------------------------------------
;Acknowledge senden
i2c_wAck:
		scl0
		sda0
		rcall	i2c_wait
		scl1
		Wstate
		rcall	i2c_wait
		scl0
		rcall	i2c_wait
		ret
;-------------------------------------------------------------------------------------
;No-Acknowledge senden
i2c_wNAck:
		scl0
		sda1
		rcall	i2c_wait
		scl1
		WState
		rcall	i2c_wait
		scl0
		rcall	i2c_wait
		ret
;-------------------------------------------------------------------------------------
;Acknowledge vom Slave
i2c_ReadACK:
		sda1
		scl1
		WState
		rcall	i2c_wait
		sbis	i2cPort,SDA
		rjmp	NoAckS
		set					;T-Flag =1, Fehler
		rjmp	Ack_ex
NoAckS:	clt					;T-Flag auf 0 = alles klar
Ack_ex:	scl0
		rcall	i2c_wait
		ret
;-------------------------------------------------------------------------------------
;Byte (temp, r16) schreiben, Ack lesen -> T-Flag
i2c_wByte:
		scl0
		sec					;Flag fr bertragungsende (Anstatt Schleifenzhler)
		rol		temp		;Carry ins Byte, MSB -> Carry
		rjmp	bit1w		;Bit1 senden
NxtBit1:lsl		temp		;Bei den Restlichen Bits Nullen ins Byte schieben
		breq	i2c_ex		;Das letzte Bit nicht senden, das ist mein Carry
bit1w:	brcc	SetBit0		;Bei 0 SDA auf 0
		sda1				;Bit ist eine 1
		rjmp	SetBit1
SetBit0:sda0		
SetBit1:nop
		scl1				;an SCL klingeln
		WState
		rcall	i2c_wait
		scl0
		rcall	i2c_wait
		rjmp	NxtBit1		;weiter zum nchsten Bit
i2c_ex:	rjmp	i2c_ReadACK	;Ack vom Slave holen und ret
;-------------------------------------------------------------------------------------
;Byte lesen, Byte in temp (r16)
i2c_rByte:
		scl0
		sda1				;SDA = Eingang !
		ldi		temp,1		;Wenn die 1 im Carry ist, ist das Byte bertragen		
		rcall	i2c_wait
NxtBit:	scl1				;SCL auf 1 und warten
		WState
		rcall	i2c_wait
		sbic	i2cPin,sda	;ist Bit 0 oder 1 ?
		rjmp	BitIs1		;wenn Bit =1 Carry setzen und reinschieben
		clc					;bei 0 Carry lschen
		rjmp	BitIs0
BitIs1:	sec
BitIs0:	scl0
		rcall	i2c_wait
		rol		temp		;ist meine 1 schon da ?
		brcc	NxtBit		;n, weiter
		ret
;-------------------------------------------------------------------------------------
;Byte lesen und Ack senden Byte in temp (r16)
i2c_rByte_ACK:
		rcall	i2c_rByte
		rjmp	i2c_wAck	
;-------------------------------------------------------------------------------------
;Byte lesen und NAck senden Byte in temp (r16)
i2c_rByte_NACK:
		rcall	i2c_rByte
		rjmp	i2c_wNAck	
;-------------------------------------------------------------------------------------
;5 s warten (8MHz) 
i2c_wait:
		rcall	delay	;3
i2c_w4:	rcall	delay	;3
		rcall	delay	;3
		rcall	delay	;3
		rcall	delay	;3
		rcall	delay	;3
delay:	nop				;1
		ret				;4
;Genau 40 Zyklen, (40/8.000.000 = 5s)
;------------------------------------------------------------------------
;I2C-Bus Routinen aufrufen

tempe:	rcall	i2c_start		;LM75 Adresse und Adresszeiger auf 0 (Temperaturregister)
		ldi		temp,LM75		;LM75 schreiben
		rcall	i2c_wByte
		brts	err1
		clr		temp			;Adresspointer auf Temperaturregister (Adr.0)
		rcall	i2c_wByte
		brts	err1
		rcall	i2c_start		;Gleich noch ein Start hinterher
		ldi		temp,LM75+1		;LM75 lesen
		rcall	i2c_wByte
		brts	err1
		rcall	i2c_rByte_ACK	;Temperatur lesen (High)
		push	temp			;erst noch LOW holen
		rcall	i2c_rByte_NACK	;(LOW)
		rcall	i2c_stop		;I2C-Bus freigeben
;----------------------------------------------------------------
;Das Temperaturregister des LM75. Die 9 Bits liegen linksbndig in einem 16-Bit Register.
;Vorzeichen (DB8) beachten! DB0 = 1/2 Grad (.0 oder .5)
;-----------------------------------------------------------------
; <---------   High   ---------->|<------------ Low ------------>
;|DB8|DB7|DB6|DB5|DB4|DB3|DB2|DB1|DB0| X | X | X | X | X | X | X |
; <---------  temp1  ----------->|<------------ temp ----------->
;-----------------------------------------------------------------
		pop		temp1			;High-Byte nach temp1, Low-Byte ist in temp
		lsl		temp			;Kommastelle (DB0) ins Carry
		rol		temp1			;Vorzeichen (DB8) ins Carry, Kommastelle -> Temp1
		clr		temp			;Die X im Lowbyte lschen
		rol		temp			;DB8 -> Temp
;-------------------------------------------------------------------
;Jetzt sieht das ganze so aus, 
; <----------   Low    ----------->|<------------ High ------------>
;|DB7|DB6|DB5|DB4|DB3|DB2|DB1| DB0 | 0 | 0 | 0 | 0 | 0 | 0 | 0 |DB8|
; <-----------  temp1  ----------->|<------------ temp ------------>
;-------------------------------------------------------------------
err1:	ret
	

text:	.db	"Auswahl         ",0
;text:	.db	"AVR-Projekte.DE",0
text1:	.db	"<Start>   Setup ",0
text2:	.db	" Start   <Setup>",0
text3:	.db	"Hochfahren ... ",0
text4:	.db	"UPM:",0,0
text5:	.db	"Schalttemperatur",0
text6:	.db	"Motorspeed",0
text7:	.db	"PWM:",0
text8:	.db	"Erinnerung MM:SS",0
text9:	.db	"Zeit:",0

chars:.db	0b11111,\
			0b00000,\
			0b00000,\
			0b00000,\
			0b00000,\
			0b00000,\
			0b00000,\
			0b11111

		.db	0b11111,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b11111

		.db	0b11111,\
			0b11000,\
			0b11000,\
			0b11000,\
			0b11000,\
			0b11000,\
			0b11000,\
			0b11111

		.db	0b11111,\
			0b11100,\
			0b11100,\
			0b11100,\
			0b11100,\
			0b11100,\
			0b11100,\
			0b11111

		.db	0b11111,\
			0b11110,\
			0b11110,\
			0b11110,\
			0b11110,\
			0b11110,\
			0b11110,\
			0b11111

		.db	0b00000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b10000,\
			0b00000

		.db	0b00000,\
			0b10001,\
			0b11001,\
			0b11101,\
			0b11111,\
			0b11101,\
			0b11001,\
			0b10000

		.db	0b00000,\
			0b10001,\
			0b11001,\
			0b10101,\
			0b10011,\
			0b10101,\
			0b11001,\
			0b10000
.dseg	
.org		SRAM_START
stunden:	.byte	1	;Uhr
minuten:	.byte	1
sekunden:	.byte	1
msekunden:	.byte	1
amin:		.byte	1	;alarm
asek:		.byte	1
switch:		.byte	1
wtemp:		.byte	1	;Temperatur verzgerung
startspeed:	.byte	1	;Motorspeed
stemp:		.byte	1	;Temperatur
wrel:		.byte	1	;Relais verzgerung

